/* -------------------------------------------------------------------------
                Copyright (c) enGenic 2001.
                All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef BRD_GENERIC_H
#define BRD_GENERIC_H

#include <string>
using std::string;
#include <map>
using std::map;

#include "..\escutil\export_declarations.h"
#include "..\escutil\ESCTime.h"
#include "..\TFServer\ivrinfo.h"
#include "..\escutil\parmlist.h"
#include "..\inet\HTTPTransaction.h"
#include "..\inet\SDPInfo.h"
#include "..\TFEngine\step_dictate.h"
#include "..\Languages\LanguageModule.h"
#include "Board_communication.h"

#define VOL_OFFSET       0
#define VOL_MULTIPLIER   7

#define AUDIO_POSITION_START       1
#define AUDIO_POSITION_END         2
#define AUDIO_POSITION_FORWARD     3
#define AUDIO_POSITION_BACKWARD    4

#define AUDIO_SPEED_INCREASE       1
#define AUDIO_SPEED_DECREASE       2

#define AUDIO_VOLUME_NORMAL        1
#define AUDIO_VOLUME_INCREASE      2

/* -------------------------------------------------------------------------
   Class:       CCriticalSection
   ------------------------------------------------------------------------- */
class CCriticalSection
{
public:
   CCriticalSection() : bInCS(false) {}
   static void Create() { InitializeCriticalSection (&cs); }
   static void Delete() { DeleteCriticalSection (&cs); }
   void Enter() { if (!bInCS) { EnterCriticalSection (&cs); bInCS = true;  } }
   void Leave() { if ( bInCS) { LeaveCriticalSection (&cs); bInCS = false; } }
private:
   static CRITICAL_SECTION  cs;
          bool              bInCS;
};

// This number should be the same in brd_XXX.dll, AppBuilder, and TF_Monitor.
#define MAX_LINES  300

// Modified to support Dialogic digital model - TF Oct.25/2001
/* WAS:
typedef struct
{
   bool             inUse;
   unsigned         board;               // AG Board number.
   unsigned char    mvip_stream;         // MVIP stream.
   unsigned char    mvip_slot;           // MVIP timeslot.
   int              channel;
} BRD_LINE;
*/

typedef struct                        //  NMS               Dialogic
{                                     //  ---------------   ---------------
   bool           inUse;              //  in use flag       in use flag
   // voice devices
   char           boardName[10];      //  n/a               voice board name
   int            board;              //  board number      voice board #
   int            channel;            //  channel #         voice channel #
   int            chdev;              //  n/a               channel device handle
   // trunk type
   unsigned char  trunkType;          //  n/a               trunk type (Analog, T1, E1)
   // digital trunk devices
   char           trunkName[10];      //  n/a               trunk board name
   unsigned char  stream;             //  MVIP stream       trunk board #
   unsigned char  slot;               //  MVIP slot         timeslot
   int            tsdev;              //  n/a               timeslot device handle
} BRD_LINE;

typedef struct
{
   int frequency1;
   int frequency2;
   int minimumOnTime;
   int maximumOnTime;
   int minimumOffTime;
   int maximumOffTime;
} ToneCadenceParameters;

/* -------------------------------------------------------------------------
   Class:       BoardEvent

   Purpose:     Every specific board DLL must implement this class.

                This generic definition allows TeleFlow Server to load any
                DLL that implements a class that derives from this class.
   ------------------------------------------------------------------------- */
class BoardEvent
{
	public:
   	BoardEvent(void);
   	virtual ~BoardEvent(void);

      virtual bool SameEvent(BoardEvent* comparisonEvent) = 0;
};

// Used in the streamClients list in AudioState
// Holds the socket being streamed to and the ID of the client thread that
// requested streaming
typedef struct StreamerClientInfo
{
   // Socket to stream audio to
   // Socket is closed (handle is ACE_INVALID_HANDLE) when streaming is paused
   UDPSocket* socket;

   // Pointer to communication TCPSocket for streaming client, used to identify
   // the client managed by the TCPClientManager in order to send messages back to TFListener
   // Do not use directly, only as an identifier!
   TCPSocket* client;
} StreamerClientInfo;

// Designed as a replacement for AUDIO_CTRL
// Holds state information about audio processes on a given port
// May be used as a base class for boards that need to store more information
// The voice recognition portion has not yet been used
class AudioState
{
   public:

      AudioState(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub);
      virtual ~AudioState(void);

      // Sets ActivateBargeIn() to return true
      // Sets IsRecording() to return true
      // Returns true if successful
      bool ActivateBargeIn(void);

      // Stores destination and associated TCPClient's ID in list of clients that
      // are sent audio by StreamAudio()
      // Returns true if successful
      bool AddStreamer(ACE_INET_Addr destination, TCPSocket* client);

      // Sets ActivateBargeIn() to return false
      // Note that it does not change the return value of IsRecording()
      // Returns true if successful
      bool DeactivateBargeIn(void);

      // Returns the name of the file being recorded
      const char* GetRecordFileName(void);

      // Returns true if barge-in functionality is active
      // Note that Isrecording will return true if IsBargeInActive returns true
      bool IsBargeInActive(void);

      // Returns true if barge-in is permitted when possible
      // e.g. off-hook, not placing a call, and not recording
      bool IsBargeInPermitted(void);

      // Returns true if in dictation mode
      bool IsDictating(void);

      // Returns true if the current audio process is interruptable by touch tone
      bool IsInterruptable(void);

      // Returns true if paused in fancy play or dictation
      bool IsPaused(void);

      // Returns true if playing audio
      bool IsPlaying(void);

      // Returns true if recording audio
      bool IsRecording(void);

      // Returns true if hardware is streaming audio
      bool IsStreaming(void);

      // Returns true if streaming audio has been set to permitted by PermitStreaming()
      bool IsStreamingPermitted(void);

      // Sets IsPaused() to return true
      void Pause(void);

      // Stops sending audio to client specified by destination but retains the
      // ID of the client thread that requested streaming for notification by
      // NotifyStreamClients()
      bool PauseStreamer(ACE_INET_Addr destination);

      // Stops sending audio to all clients that are being sent audio by StreamAudio()
      // but retains the IDs of the client threads that requested streaming for
      // notification by NotifyStreamClients()
      void PauseStreamers(void);

      void PermitBargeIn(bool permit);

      // Allows streaming of audio with true and dis-allows streaming with false
      void PermitStreaming(bool permission = true);

      // Returns the playing format
      SOUND_FORMAT PlayFormat(void);

      // Returns the recording format
      SOUND_FORMAT RecordFormat(void);

      bool ReactivateBargeIn(void);

      void ReactivateBargeIn(bool state);

      // Removes destination from the list of clients that are sent audio by StreamAudio()
      // Sends a TCPCLIENT_EVT_STOP_STREAM message to the client thread that
      // requested streaming
      bool RemoveStreamer(ACE_INET_Addr destination);

      // Empties the list of clients that are sent audio by StreamAudio()
      void RemoveStreamers(void);

      virtual void Reset(void);

      // Returns true if streaming should restart when it next ends, such as at
      // the end of a recording that uses streaming
      bool RestartStreaming(void);

      // Sets RestartStreaming() to return state
      void RestartStreaming(bool state);

      // Sets IsInterruptable() to return the value interruptable
      void SetInterruptable(bool interruptable = true);

      // Sets IsInterruptable() to return false
      void SetUninterruptable(void);

      // Notifies that the last play started with StartPlaying() finished to completion
      // Sets playedEntireAudio to true
      void SetPlayedEntireAudio(void);

      // Returns the current setting of playedEntireAudio
      bool GetPlayedEntireAudio(void);

      // Returns the time the most recent Play started
      ESCTime GetPlayStartTime();

      // Returns the time the most recent Play ended
      // What should it return if it's invalid??
      ESCTime GetPlayEndTime();

      // Sets IsDictating() to return true
      // Returns true if successful
      bool StartDictating(void);

      // Stores play format (which can be retrieved by PlayFormat())
      // Sets IsPlaying() to return true
      // Sets IsPaused() to return false
      // Returns true if successful
      bool StartPlaying(SOUND_FORMAT format);

      // Stores play format (which can be retrieved by PlayFormat())
      // Stores recording file name (which can be retrieved by GetRecordFileName())
      // Sets IsRecording() to return true
      // Sets IsPaused() to return false
      // Returns true if successful
      virtual bool StartRecording(const char* fileName, SOUND_FORMAT format);

      // Initialises socket to stream audio to destination and sends a start
      // message with format to the associated TCP client
      // Returns true if successful
      bool StartStreamer(ACE_INET_Addr destination, SOUND_FORMAT format);

      // Initialises sockets to stream audio and sends start messages with format
      // to the associated TCP clients of all destinations in list of clients
      // that are sent audio by StreamAudio()
      void StartStreamers(SOUND_FORMAT format);

      // Sets IsStreaming() to return true
      // Sets RestartStreaming() to return true (same as RestartStreaming(true))
      // Returns true if successful
      bool StartStreaming(void);

      // Sets IsDictating() to return false
      void StopDictating(void);

      // Sets IsPlaying() to return false
      // Sets IsPaused() to return false
      void StopPlaying(void);

      // Sets IsRecording() to return false
      // Sets IsPaused() to return false
      virtual void StopRecording(void);

      // Makes IsStreaming() return false
      void StopStreaming(void);

      // Sends the buffer of audio data to all destinations in the client list
      void StreamAudio(const char* buffer, int bufferSize);

      // Returns the number of clients in the list of clients that are sent
      // audio by StreamAudio()
      int Streamers(void);

      // Returns the streaming format
      SOUND_FORMAT StreamFormat(void);

      // Sets IsPaused() to return false
      void UnPause(void);

   protected:

      SOUND_FORMAT playFormat;
      SOUND_FORMAT streamFormat;
      SOUND_FORMAT recordFormat;
      string recordFileName;

      map<ACE_INET_Addr, StreamerClientInfo> streamClients;
      Mutex streamClientsMutex;
      CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub;

      bool isBargeInActive;
      bool isBargeInPermitted;
      bool isDictating;
      bool isInterruptable;
      bool isPaused;
      bool isPlaying;
      bool isRecording;
      bool isStreaming;
      bool isStreamingPermitted;
      bool reactivateBargeIn;
      bool restartStreaming;
      ESCTime   playStartTime;
      ESCTime   playEndTime;
      bool      playedEntireAudio;
};

/* -------------------------------------------------------------------------
   Class:       BoardGeneric

   Purpose:     Every specific board DLL must implement these functions.

                This generic definition allows TeleFlow Server to load any
                DLL that implements a class that derives from this class.
   ------------------------------------------------------------------------- */
class BoardGeneric
{
	public:

   	// Object state data types.
	   enum callMode { newLine, existingLine };

		// Sean: Apparently unnecessary code but used in brd_dialogic_code.*
   	enum state { good, bad };

	   enum TTSTypes { TTS_FILE_TYPE, TTS_STRING_TYPE };

      // TeleFlow Trunk type defines - TF 2005/08/05
	   enum TFTrunkType { TF_TRUNK_TYPE_INVALID, TF_TRUNK_TYPE_NONE,
                         TF_TRUNK_TYPE_SIMULATOR,
                         TF_TRUNK_TYPE_ANALOG,
                         TF_TRUNK_TYPE_T1, TF_TRUNK_TYPE_E1,
                         TF_TRUNK_TYPE_VOIP,
                         TF_TRUNK_TYPE_STATION };

      enum ISDNNumberPlan { UNKNOWN_NUMBER_PLAN = 0x00,       // unknown plan
                            ISDN_NUMBER_PLAN = 0x01,          // ISDN numbering plan (CCITT E.164)
                            TELEPHONY_NUMBER_PLAN = 0x02,     // Telephony numbering plan (CCITT E.163)
                            PRIVATE_NUMBER_PLAN = 0x09 };     // private numbering plan

      enum ISDNNumberType { UNKNOWN_NUMBER_TYPE = 0x00,       // unknown type
                            INTERNATIONAL_NUMBER_TYPE = 0x01, // international number
                            NATIONAL_NUMBER_TYPE = 0x02,      // national number
                            LOCAL_NUMBER_TYPE = 0x04 };       // AT5 local (directory) number

      SDPInfo sdp;
      
	   // ---- CONSTRUCTOR / DESTRUCTOR -------------------------------------------
		// BoardGeneric(void);
		// BoardGeneric(PIVRINFO pInfo);
		// BoardGeneric(Log* outLog, int appId, const char* appName);
		// BoardGeneric(const BoardGeneric&);
		// BoardGeneric& operator=(const BoardGeneric&);
	   BoardGeneric(PIVRINFO pIVRInfo);
   	virtual ~BoardGeneric(void);

	   // ---- CLASS FUNCTIONS ----------------------------------------------------
      virtual void Reset(void);
      virtual bool GetLineOnHook(void) const;

      // returns the preferred audio format for this instance of a board class
      virtual SOUND_FORMAT GetPreferredFormat(void) const;

	   virtual const char* GetTrunkTypeString(void) const;
	   virtual const char* GetTrunkTypeString(TFTrunkType inputType) const;
	   virtual IV_STATUS WaitWhilePaused(void);
      virtual IV_STATUS ProcessSystemEvent(void);
      virtual ESCTime GetCallStartTime(void) const;
      virtual ESCTime GetCallEndTime(void) const;
      virtual float GetCallElapsedTime(void) const;
      virtual IV_STATUS DisableReorderToneDetection(void);
      virtual IV_STATUS EnableReorderToneDetection(void);
      virtual IV_STATUS StartReorderToneDetection(void) = 0;
      virtual IV_STATUS StopReorderToneDetection(void) = 0;
      virtual IV_STATUS SetReorderTone(int frequency1, int frequency2, int minimumOnTime,
         int maximumOnTime, int minimumOffTime, int maximumOffTime);
      virtual IV_STATUS Wait(int timeout, bool interruptable) = 0;
      virtual float GetPlayElapsedTime(void) const;
      virtual bool GetPlayCompleted(void) const;
      virtual WORD GetCallReference(void);

	   // ---- PARAMETER FUNCTIONS ------------------------------------------------
   	virtual IV_STATUS SetParameter(const char* pszParm, DWORD dwParmValue);

	   // ---- LINE FUNCTIONS -----------------------------------------------------
   	virtual IV_STATUS BlockCalls(void);

      // timeout of 0 inidicates returns immediately with IVR_SUCCESS if an event
      // has already occurred or IVR_TIMEOUT if no event has occurred
      // timeout of FOREVER inidicates wait until event arrives
      virtual IV_STATUS CallTrigger(const char* triggerList, const char*& trigger, char*& triggerInfo, long timeout);

      // flashTime of 0 skips flash
   	virtual IV_STATUS FlashHook(long flashTime) = 0;

	   virtual IV_STATUS Hangup(bool returnHangup = false, char* pszCause = NULL) = 0;
		virtual IV_STATUS Pickup(void) = 0;
	   virtual IV_STATUS PlaceCall(const char* digits, const char* callerID, int numrings,
         callMode mode, int flashdur, bool connectOnCED, bool connectOnSIT,
         int* pVoiceLength, int silenceTimeout, int maxVoiceTimeout,
         ISDNNumberPlan callingNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType callingNumType = UNKNOWN_NUMBER_TYPE,
         ISDNNumberPlan calledNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType calledNumType = UNKNOWN_NUMBER_TYPE) = 0;
	   virtual IV_STATUS UnblockCalls(void);
   	virtual IV_STATUS WaitForCall(long timeout, int numrings, bool pickup,
         char* calledDigits, char* callingDigits, char* handsetDisplay, int numdigits = 0) = 0;
	   virtual IV_STATUS TransferCall(WORD otherCallReference);

	   // ---- SWITCHING FUNCTIONS ------------------------------------------------
      virtual IV_STATUS Switch(int newPort, bool bListen);
      virtual IV_STATUS UnSwitch(void);

	   // ---- FAX FUNCTIONS ------------------------------------------------------
   	virtual IV_STATUS FaxQueueAdd(char* fullFile);
	   virtual IV_STATUS FaxQueueClr(void);
   	virtual IV_STATUS FaxReceive(char* file, char* SID);
	   virtual IV_STATUS FaxSend(const char* phoneNo, int ringCount, const char* sendSID);

   	// ---- TT FUNCTIONS -------------------------------------------------------
		// This should return an IV_STATUS!
   	virtual IV_STATUS FlushDigits(void) = 0;
	   virtual IV_STATUS GetDigit(char& ivNextTT, int newTime);
   	virtual IV_STATUS GenerateDigits(const char* ttStr) = 0;
	   virtual IV_STATUS QueryDigit(char& digit) = 0;

      // timeout of 0 inidicates returns immediately with IVR_SUCCESS if digit
      // has already occurred or IVR_TIMEOUT if no digit has occurred
      // count must be > 0 or INFINITE
      // digits buffer is populated by GetDigits and the client is repsonible for
      // deleting the buffer
      // Digits A,B,C, and D are in upper case
   	virtual IV_STATUS GetDigits(
      	unsigned	count,     		// Number of digits to collect.
	      char*&	digits,    		// Digit buffer returned from GetDigits
   	   int		newTime,   		// Timeout between key presses.
      	const char*	tOkay,    		// Digits that are valid.
      	const char*	ttDone) = 0;	// Digits that terminate TT collection.

	   // ---- AUDIO FUNCTIONS ----------------------------------------------------
   	virtual IV_STATUS AudioBeep(bool interruptable) = 0;
	   virtual IV_STATUS AudioCopy(
   	   char *     srcFile,         // Name of source file.
      	char *     dstFile);        // Name of destination file.

	   virtual IV_STATUS AudioDelete(char* filename);

      // maxRecTime of 0 records an empty sound file
      // maxRecTime of FOREVER indicates record until interrupt or silence timeout
   	virtual IV_STATUS AudioRecord(const char* filename, bool interruptable,
         SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
         bool disableBeep, bool automaticGainControl);

		// Record to an audio buffer
		// Audio buffer must be created by the calling module
		// Maximum silence time is maxSilence or until buffer full, which ever comes first
      // There is no maximum record time, as this is determined by the size of the buffer
		virtual IV_STATUS AudioRecordBuffer(char* buffer, unsigned int bufsize,
         bool interruptable, SOUND_FORMAT format, int volLevel,
         int maxSilence, bool disableBeep, bool automaticGainControl);

      // maxRecTime of 0 records an empty or short as possible sound file
      // maxRecTime of FOREVER indicates record until interrupt or silence timeout
      virtual IV_STATUS StartRecordFile(const char* filename, bool interruptable,
         SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
         bool disableBeep, bool automaticGainControl);

		// Audio buffer must be created by the calling module
		// Maximum silence time is maxSilence or until buffer full, which ever comes first
      // There is no maximum record time, as this is determined by the size of the buffer
		virtual IV_STATUS StartRecordBuffer(char* buffer, unsigned int bufsize,
         bool interruptable, SOUND_FORMAT format, int volLevel,
         int maxSilence, bool disableBeep, bool automaticGainControl);

      virtual IV_STATUS Dictate(StepDictate::TTAction* TTActions, int silenceTimeout,
         int& exitPosition, int maximumRecordTime, int startPosition,
         const char* dictationFilename, StepDictate::START_BEHAVIOUR startBehaviour,
         bool automaticGainControl, SOUND_FORMAT format, bool disableBeep,
         StepDictate::END_BEHAVIOUR endBehaviour, int volLevel, const char* menuFilename,
         SOUND_FORMAT menuFormat, int menuTimeout, int menuRetries);

      virtual IV_STATUS AudioStopRecord(void);

      virtual IV_STATUS AudioWaitForRecordStop(void);

      // maxRecTime of 0 records an empty or short as possible sound file
      // maxRecTime of FOREVER indicates record until interrupt or silence timeout
	   virtual IV_STATUS AudioPlay(const char* filename, bool interruptable,
   		SOUND_FORMAT format, int volLevel, bool fancy, unsigned int playFromTime = 0,
         unsigned int playForTime = FOREVER);

	   virtual IV_STATUS AudioPlayBuffer(char* audioBuffer, unsigned bufferSize,
         bool interruptable, SOUND_FORMAT format, int volLevel);

      // playForTime of 0 indicates do not play file
      // playForTime of FOREVER indicates play until end of file
	   virtual IV_STATUS StartPlayFile(const char* filename, bool interruptable,
   		SOUND_FORMAT format, int volLevel, unsigned int playFromTime = 0, unsigned int playForTime = FOREVER);

      virtual IV_STATUS StartPlayBuffer(char* audioBuffer, unsigned bufferSize,
         bool interruptable, SOUND_FORMAT format, int volLevel);

	   virtual IV_STATUS AudioStopPlay(void);

      virtual IV_STATUS AudioWaitForPlayStop(void);

      virtual IV_STATUS FancyPlayFileControl(void);

	   virtual IV_STATUS Say(const char* text, bool interruptable, const char* langDir,
   	   int volLevel, LanguageModule::SAY_FORMAT spkFormat);

	   virtual IV_STATUS AudioRecognise(int noVoiceTimeout, int endOfVoiceTimeout) = 0;
      virtual SOUND_FORMAT GetSRAudioFormat(void);
      virtual IV_STATUS PermitBargeIn(bool permit);
      virtual IV_STATUS ActivateBargeIn(void);
      virtual IV_STATUS DeactivateBargeIn(void);
      virtual void SetSREndOfSpeechTimeout(unsigned int timeout);
      virtual unsigned int GetSREndOfSpeechTimeout(void);

      virtual IV_STATUS AudioPlayEvalMessage(void) = 0;

      virtual IV_STATUS HTTPRequest(HTTPTransaction& http, int timeout, int connectionTimeout,
         int connectionRetries, const char* filename, SOUND_FORMAT format, int volLevel);

      virtual IV_STATUS AddStreamer(ACE_INET_Addr destination, TCPSocket* client);
      virtual IV_STATUS RemoveStreamers(void);
      virtual IV_STATUS RemoveStreamer(ACE_INET_Addr destination);
      virtual IV_STATUS PauseStreamers(void);
      virtual IV_STATUS PauseStreamer(ACE_INET_Addr destination);
      virtual IV_STATUS StartStreaming(SOUND_FORMAT format);
      virtual IV_STATUS StopStreaming(void);

	   // ---- CONFERENCING FUNCTIONS -------------------------------------------
      virtual IV_STATUS ConfCreate(DWORD iMaxSeats,    // maximum number of seats
                           bool bEchoCancel = false,   // use Echo cancellation
                           bool bDTMFClamping = false, // use DTMF clamping
                           bool bToneClamping = false, // use Tone clamping
                           bool bCoaching = false,     // use coaching
                           int* pConfHandle = NULL);   // to receive TeleFlow conference handle

      // Destory a conference
      virtual IV_STATUS ConfDestroy(unsigned iHandle); // handle of conference to destroy

      // Join the current port's DSP to a conference
      virtual IV_STATUS ConfDSP(unsigned iHandle,      // handle of conference to join
                                bool bAutogain,        // use automatic gain control?
                                int iSetBeep);         // setting for beep on entry/exit

      // Join the current port (ie: the caller) to a conference
      virtual IV_STATUS ConfJoin(unsigned iHandle,     // handle of conference to join
                                 bool bAutogain,       // use automatic gain control
                                 int iSetBeep);        // setting for beep on entry/exit

      // Join the current port (pInfo->iPort) to a conference
      virtual IV_STATUS ConfLeave();                   // no parameters

      // Allow/disallow audio streaming
      virtual IV_STATUS PermitStreaming(bool permission);

	protected:

      AudioState* audioState;
      SOUND_FORMAT      preferredFormat;     // Preferred audio format
	   bool              lineOffHook;			// TRUE if line off hook
   	int               curLine;             // IVR port that this thread is using (-1 when not in use)
	   PIVRINFO          pInfo;               // Thread application information
   	bool              bBrdInit;            // True if board is initialized.
      TFTrunkType       trunkType;           // Trunk type of the current port (TF_TRUNK_TYPE_NONE for portless apps)
	   Log              *ivLog;               // Application log
	   char             *TTSDirectory;			// Text To Speech file directory
   	int               lastLine;				// Last IVR port on the system
	   ESCTime           callStart;           // Time at which a call is placed or answered
   	ESCTime           callEnd;             // Time at which the last hang up occurred
   	BRD_LINE          lines[MAX_LINES];		// Groundwork for IVR port independent threads?  Currently only lines[curLine] contains any useful information
   	HANDLE            hWaitTimer;          // Used for handling timeouts.
	   HANDLE            hGenericTimer;       // Used as a general timer by anyone.
   	HANDLE           *waitObjects;         // List of wait objects to monitor.
	   unsigned          numWaitObjects;      // No. of objects in 'waitObjects'.
	   CCriticalSection	cs;
      const char* speakAudioExtension;
      ToneCadenceParameters reorderTone;
      bool disableReorderToneDetection;
      unsigned int SREndOfSpeechTimeout;

      virtual IV_STATUS ProcessBoardMessage(BoardCommunicationPacket* boardMessage);
		virtual IV_STATUS ProcessBoardEvent(BoardEvent* boardEvent, bool& bFound) = 0;
   	IV_STATUS WaitForEvent(
   		BoardEvent* boardEvent,   // Event structure
	      long timeout = FOREVER); // Timeout in seconds or FOREVER to wait forever
   	IV_STATUS WaitForSpecificEvent(BoardEvent* desiredEvent, BoardEvent* boardEvent, long timeout = FOREVER);
      void SetGenericEvent(HANDLE event);
      void ClearGenericEvent(void);

      // Set the wait object handle for the board-level event queue; used by certain
      // functions (ie: CallTrigger()) to re-enable the monitoring of board events after
      // the signal had been disabled by a call to ClearBoardEvent() (see below)
      void SetBoardEvent(HANDLE event);

      // Disable the wait object handle for the board-level event queue; used by certain
      // functions to prevent board events from interrupting its operation (ie: CallTrigger()
      // has its own queue of events). The signal must be re-enabled with a call to
      // SetBoardEvent() (see above)
      void ClearBoardEvent(void);

      // Reverts one of the Waitable Timer objects to un-signaled state; deletes and recreates
      // the timer so that it will be cancelled even if it is already in the signalled state
      void RecreateTimer(HANDLE requestedTimer);

      SOUND_FORMAT GetExtensionFormat(const char* fileName);

   private:

      // All handles must be valid (and not NULL) in an array passed to WaitForMultipleObjects
      // This HANDLE is created to provide a valid handle but is never signalled
	   HANDLE hFakeGenericEvent;

};

/* -------------------------------------------------------------------------
   Purpose:     These functions must be implemented by all board DLLs.
                These functions are used to instantiate a board class
                object.
   ------------------------------------------------------------------------- */

typedef BoardGeneric* (*TCreateInstance)(PIVRINFO pInfo);
typedef void (*TReleaseInstance)(BoardGeneric *brd);

FUNCTION_DECLARATION(BoardGeneric*) CreateInstance (PIVRINFO pInfo);
FUNCTION_DECLARATION(void) ReleaseInstance (BoardGeneric *obj);

// -------------------------------------------------------------------------
// Purpose:     These functions are used to call any initialization
//              functions required by the hardware vendor's API
//              Must be implemented for all board DLLs, but can be empty.
//              TF 2004-09-24
// -------------------------------------------------------------------------

typedef bool (*TOpenHardwareDriver)(Log *sysLog);
typedef bool (*TCloseHardwareDriver)(Log *sysLog);
FUNCTION_DECLARATION(bool) OpenHardwareDriver (Log *sysLog);
FUNCTION_DECLARATION(bool) CloseHardwareDriver (Log *sysLog);

#endif
